structure Ast =
struct

  exception RunTimeTypeError
  exception TermIsValue
  exception InternalError

  datatype exp = 
    NumExp of int
  | BoolExp of bool
  | TimesExp of exp * exp 
  | DivExp of exp * exp
  | PlusExp of exp * exp
  | SubExp of exp * exp
  | ParenExp of exp
  | IfExp of exp * exp * exp
  | EqExp of exp * exp
  | GTExp of exp * exp
  

  (** Implement your big-step semantics **)
  fun evalBigStep e = raise RunTimeTypeError

  (** Implement your small-step semantics **)
  fun evalSmallStep e = raise RunTimeTypeError

  fun toPrettyString e = 
    let
	fun mkString_two s e1 e2 = 
	    "(" ^ toPrettyString e1 ^  s ^  toPrettyString e2 ^ ")"
    in
	case e of
	    NumExp n =>  "Num " ^ (Int.toString n)
	  | BoolExp f =>  "Bool " ^ (Bool.toString f)
	  | TimesExp (e1,e2) => mkString_two "*" e1 e2
	  | DivExp   (e1,e2) => mkString_two "/" e1 e2
	  | PlusExp  (e1,e2) => mkString_two "+ " e1 e2
	  | SubExp   (e1,e2) => mkString_two "- " e1 e2
	  | ParenExp e1 =>  "(" ^ toPrettyString e1 ^ ")"
          | IfExp (e1,e2,e3) => "If " ^ toPrettyString e1 ^ " then " ^
			        toPrettyString e2 ^ " else "  ^ toPrettyString e3 ^ " "
	  | GTExp   (e1,e2) => mkString_two ">" e1 e2
	  | EqExp   (e1,e2) => mkString_two "=" e1 e2
    end


  fun toString e = 
    let
	fun mkString_two s e1 e2 = 
	    s ^ "(" ^ (toString e1) ^ "," ^ (toString e2) ^ ")"

	fun mkString_three s e1 e2 e3 = 
	    s ^ "(" ^ (toString e1) ^ "," ^ (toString e2) ^  (toString e2) ^ ")"

	fun mkString_one s e = s ^ "(" ^ (toString e) ^ ")"
    in
	case e of
	    NumExp n =>  ("Num " ^ Int.toString n ^ "\n")
	  | BoolExp n =>  ("Bool " ^ Bool.toString n ^ "\n")
	  | TimesExp (e1,e2) => mkString_two "Times " e1 e2
	  | DivExp   (e1,e2) => mkString_two "Div " e1 e2
	  | PlusExp  (e1,e2) => mkString_two "Plus " e1 e2
	  | SubExp   (e1,e2) => mkString_two "Sub " e1 e2
	  | ParenExp e1 => mkString_one "Paren " e1
          | IfExp (e1,e2,e3) => mkString_three "If" e1 e2 e3
	  | GTExp   (e1,e2) => mkString_two "GT " e1 e2
	  | EqExp   (e1,e2) => mkString_two "EQ " e1 e2
    end
end
